@@ -1,43 +1,38 @@
-import os, os.path, unittest
-from django.contrib.gis.gdal import DataSource, Envelope, OGRGeometry, OGRException, OGRIndexError
-from django.contrib.gis.gdal.field import OFTReal, OFTInteger, OFTString
-from django.contrib import gis
+import os
+import unittest
+from unittest import skipUnless
 
-# Path for SHP files
-data_path = os.path.join(os.path.dirname(gis.__file__), 'tests' + os.sep + 'data')
-def get_ds_file(name, ext):
-    return os.sep.join([data_path, name, name + '.%s' % ext])
+from django.contrib.gis.gdal import HAS_GDAL
+from django.contrib.gis.geometry.test_data import get_ds_file, TestDS, TEST_DATA
 
-# Test SHP data source object
-class TestDS:
-    def __init__(self, name, **kwargs):
-        ext = kwargs.pop('ext', 'shp')
-        self.ds = get_ds_file(name, ext)
-        for key, value in kwargs.items():
-            setattr(self, key, value)
+if HAS_GDAL:
+    from django.contrib.gis.gdal import DataSource, Envelope, OGRGeometry, OGRException, OGRIndexError, GDAL_VERSION
+    from django.contrib.gis.gdal.field import OFTReal, OFTInteger, OFTString
 
-# List of acceptable data sources.
-ds_list = (TestDS('test_point', nfeat=5, nfld=3, geom='POINT', gtype=1, driver='ESRI Shapefile',
-                  fields={'dbl' : OFTReal, 'int' : OFTInteger, 'str' : OFTString,},
-                  extent=(-1.35011,0.166623,-0.524093,0.824508), # Got extent from QGIS
-                  srs_wkt='GEOGCS["GCS_WGS_1984",DATUM["WGS_1984",SPHEROID["WGS_1984",6378137,298.257223563]],PRIMEM["Greenwich",0],UNIT["Degree",0.017453292519943295]]',
-                  field_values={'dbl' : [float(i) for i in range(1, 6)], 'int' : range(1, 6), 'str' : [str(i) for i in range(1, 6)]},
-                  fids=range(5)),
-           TestDS('test_vrt', ext='vrt', nfeat=3, nfld=3, geom='POINT', gtype='Point25D', driver='VRT',
-                  fields={'POINT_X' : OFTString, 'POINT_Y' : OFTString, 'NUM' : OFTString}, # VRT uses CSV, which all types are OFTString.
-                  extent=(1.0, 2.0, 100.0, 523.5), # Min/Max from CSV
-                  field_values={'POINT_X' : ['1.0', '5.0', '100.0'], 'POINT_Y' : ['2.0', '23.0', '523.5'], 'NUM' : ['5', '17', '23']},
-                  fids=range(1,4)),
-           TestDS('test_poly', nfeat=3, nfld=3, geom='POLYGON', gtype=3, 
-                  driver='ESRI Shapefile',
-                  fields={'float' : OFTReal, 'int' : OFTInteger, 'str' : OFTString,},
-                  extent=(-1.01513,-0.558245,0.161876,0.839637), # Got extent from QGIS
-                  srs_wkt='GEOGCS["GCS_WGS_1984",DATUM["WGS_1984",SPHEROID["WGS_1984",6378137,298.257223563]],PRIMEM["Greenwich",0],UNIT["Degree",0.017453292519943295]]'),
-           )
+    # List of acceptable data sources.
+    ds_list = (
+        TestDS('test_point', nfeat=5, nfld=3, geom='POINT', gtype=1, driver='ESRI Shapefile',
+            fields={'dbl': OFTReal, 'int': OFTInteger, 'str': OFTString},
+            extent=(-1.35011, 0.166623, -0.524093, 0.824508),  # Got extent from QGIS
+            srs_wkt='GEOGCS["GCS_WGS_1984",DATUM["WGS_1984",SPHEROID["WGS_1984",6378137,298.257223563]],PRIMEM["Greenwich",0],UNIT["Degree",0.017453292519943295]]',
+            field_values={'dbl': [float(i) for i in range(1, 6)], 'int': list(range(1, 6)), 'str': [str(i) for i in range(1, 6)]},
+            fids=range(5)),
+        TestDS('test_vrt', ext='vrt', nfeat=3, nfld=3, geom='POINT', gtype='Point25D', driver='VRT',
+            fields={'POINT_X': OFTString, 'POINT_Y': OFTString, 'NUM': OFTString},  # VRT uses CSV, which all types are OFTString.
+            extent=(1.0, 2.0, 100.0, 523.5),  # Min/Max from CSV
+            field_values={'POINT_X': ['1.0', '5.0', '100.0'], 'POINT_Y': ['2.0', '23.0', '523.5'], 'NUM': ['5', '17', '23']},
+            fids=range(1, 4)),
+        TestDS('test_poly', nfeat=3, nfld=3, geom='POLYGON', gtype=3,
+            driver='ESRI Shapefile',
+            fields={'float': OFTReal, 'int': OFTInteger, 'str': OFTString},
+            extent=(-1.01513, -0.558245, 0.161876, 0.839637),  # Got extent from QGIS
+            srs_wkt='GEOGCS["GCS_WGS_1984",DATUM["WGS_1984",SPHEROID["WGS_1984",6378137,298.257223563]],PRIMEM["Greenwich",0],UNIT["Degree",0.017453292519943295]]'),
+    )
 
-bad_ds = (TestDS('foo'),
-          )
+bad_ds = (TestDS('foo'),)
 
+
+@skipUnless(HAS_GDAL, "GDAL is required")
 class DataSourceTest(unittest.TestCase):
 
     def test01_valid_shp(self):
@@ -63,7 +58,7 @@ class DataSourceTest(unittest.TestCase):
                 pass
             else:
                 self.fail('Expected an IndexError!')
-                        
+
     def test02_invalid_shp(self):
         "Testing invalid SHP files for the Data Source."
         for source in bad_ds:
@@ -71,12 +66,11 @@ class DataSourceTest(unittest.TestCase):
 
     def test03a_layers(self):
         "Testing Data Source Layers."
-        print "\nBEGIN - expecting out of range feature id error; safe to ignore.\n"
         for source in ds_list:
             ds = DataSource(source.ds)
 
             # Incrementing through each layer, this tests DataSource.__iter__
-            for layer in ds:                
+            for layer in ds:
                 # Making sure we get the number of features we expect
                 self.assertEqual(len(layer), source.nfeat)
 
@@ -84,17 +78,24 @@ class DataSourceTest(unittest.TestCase):
                 self.assertEqual(source.nfld, layer.num_fields)
                 self.assertEqual(source.nfld, len(layer.fields))
 
-                # Testing the layer's extent (an Envelope), and it's properties
-                self.assertEqual(True, isinstance(layer.extent, Envelope))
-                self.assertAlmostEqual(source.extent[0], layer.extent.min_x, 5)
-                self.assertAlmostEqual(source.extent[1], layer.extent.min_y, 5)
-                self.assertAlmostEqual(source.extent[2], layer.extent.max_x, 5)
-                self.assertAlmostEqual(source.extent[3], layer.extent.max_y, 5)
+                # Testing the layer's extent (an Envelope), and its properties
+                if source.driver == 'VRT' and (GDAL_VERSION >= (1, 7, 0) and GDAL_VERSION < (1, 7, 3)):
+                    # There's a known GDAL regression with retrieving the extent
+                    # of a VRT layer in versions 1.7.0-1.7.2:
+                    #  http://trac.osgeo.org/gdal/ticket/3783
+                    pass
+                else:
+                    self.assertEqual(True, isinstance(layer.extent, Envelope))
+                    self.assertAlmostEqual(source.extent[0], layer.extent.min_x, 5)
+                    self.assertAlmostEqual(source.extent[1], layer.extent.min_y, 5)
+                    self.assertAlmostEqual(source.extent[2], layer.extent.max_x, 5)
+                    self.assertAlmostEqual(source.extent[3], layer.extent.max_y, 5)
 
                 # Now checking the field names.
                 flds = layer.fields
-                for f in flds: self.assertEqual(True, f in source.fields)
-                
+                for f in flds:
+                    self.assertEqual(True, f in source.fields)
+
                 # Negative FIDs are not allowed.
                 self.assertRaises(OGRIndexError, layer.__getitem__, -1)
                 self.assertRaises(OGRIndexError, layer.__getitem__, 50000)
@@ -114,8 +115,7 @@ class DataSourceTest(unittest.TestCase):
                         # the feature values here while in this loop.
                         for fld_name in fld_names:
                             self.assertEqual(source.field_values[fld_name][i], feat.get(fld_name))
-        print "\nEND - expecting out of range feature id error; safe to ignore."
-                        
+
     def test03b_layer_slice(self):
         "Test indexing and slicing on Layers."
         # Using the first data-source because the same slice
@@ -132,7 +132,9 @@ class DataSourceTest(unittest.TestCase):
             self.assertEqual(control_vals, test_vals)
 
     def test03c_layer_references(self):
-        "Test to make sure Layer access is still available without the DataSource."
+        """
+        Ensure OGR objects keep references to the objects they belong to.
+        """
         source = ds_list[0]
 
         # See ticket #9448.
@@ -146,7 +148,10 @@ class DataSourceTest(unittest.TestCase):
         # Making sure we can call OGR routines on the Layer returned.
         lyr = get_layer()
         self.assertEqual(source.nfeat, len(lyr))
-        self.assertEqual(source.gtype, lyr.geom_type.num)        
+        self.assertEqual(source.gtype, lyr.geom_type.num)
+
+        # Same issue for Feature/Field objects, see #18640
+        self.assertEqual(str(lyr[0]['str']), "1")
 
     def test04_features(self):
         "Testing Data Source Features."
@@ -169,8 +174,9 @@ class DataSourceTest(unittest.TestCase):
                         self.assertEqual(True, isinstance(feat[k], v))
 
                     # Testing Feature.__iter__
-                    for fld in feat: self.assertEqual(True, fld.name in source.fields.keys())
-                        
+                    for fld in feat:
+                        self.assertEqual(True, fld.name in source.fields.keys())
+
     def test05_geometries(self):
         "Testing Geometries from Data Source Features."
         for source in ds_list:
@@ -187,7 +193,11 @@ class DataSourceTest(unittest.TestCase):
 
                     # Making sure the SpatialReference is as expected.
                     if hasattr(source, 'srs_wkt'):
-                        self.assertEqual(source.srs_wkt, g.srs.wkt)
+                        self.assertEqual(
+                            source.srs_wkt,
+                            # Depending on lib versions, WGS_84 might be WGS_1984
+                            g.srs.wkt.replace('SPHEROID["WGS_84"', 'SPHEROID["WGS_1984"')
+                        )
 
     def test06_spatial_filter(self):
         "Testing the Layer.spatial_filter property."
@@ -202,7 +212,7 @@ class DataSourceTest(unittest.TestCase):
 
         # Setting the spatial filter with a tuple/list with the extent of
         # a buffer centering around Pueblo.
-        self.assertRaises(ValueError, lyr._set_spatial_filter, range(5))
+        self.assertRaises(ValueError, lyr._set_spatial_filter, list(range(5)))
         filter_extent = (-105.609252, 37.255001, -103.609252, 39.255001)
         lyr.spatial_filter = (-105.609252, 37.255001, -103.609252, 39.255001)
         self.assertEqual(OGRGeometry.from_bbox(filter_extent), lyr.spatial_filter)
@@ -223,11 +233,13 @@ class DataSourceTest(unittest.TestCase):
         # should indicate that there are 3 features in the Layer.
         lyr.spatial_filter = None
         self.assertEqual(3, len(lyr))
-        
-def suite():
-    s = unittest.TestSuite()
-    s.addTest(unittest.makeSuite(DataSourceTest))
-    return s
 
-def run(verbosity=2):
-    unittest.TextTestRunner(verbosity=verbosity).run(suite())
+    def test07_integer_overflow(self):
+        "Testing that OFTReal fields, treated as OFTInteger, do not overflow."
+        # Using *.dbf from Census 2010 TIGER Shapefile for Texas,
+        # which has land area ('ALAND10') stored in a Real field
+        # with no precision.
+        ds = DataSource(os.path.join(TEST_DATA, 'texas.dbf'))
+        feat = ds[0][0]
+        # Reference value obtained using `ogrinfo`.
+        self.assertEqual(676586997978, feat.get('ALAND10'))
