@@ -8,6 +8,8 @@ a string) and returns a tuple in this format:
 """
 from __future__ import unicode_literals
 
+import functools
+from importlib import import_module
 import re
 from threading import local
 
@@ -15,19 +17,14 @@ from django.http import Http404
 from django.core.exceptions import ImproperlyConfigured, ViewDoesNotExist
 from django.utils.datastructures import MultiValueDict
 from django.utils.encoding import force_str, force_text, iri_to_uri
-from django.utils.functional import memoize, lazy
+from django.utils.functional import lazy
 from django.utils.http import urlquote
-from django.utils.importlib import import_module
 from django.utils.module_loading import module_has_submodule
 from django.utils.regex_helper import normalize
-from django.utils import six
+from django.utils import six, lru_cache
 from django.utils.translation import get_language
 
 
-_resolver_cache = {} # Maps URLconf modules to RegexURLResolver instances.
-_ns_resolver_cache = {} # Maps namespaces to RegexURLResolver instances.
-_callable_cache = {} # Maps view and url pattern names to their view functions.
-
 # SCRIPT_NAME prefixes for each thread are stored here. If there's no entry for
 # the current thread (which is the only one we ever access), it is assumed to
 # be empty.
@@ -62,7 +59,7 @@ class ResolverMatch(object):
 
     @property
     def view_name(self):
-        return ':'.join([ x for x in [ self.namespace, self.url_name ]  if x ])
+        return ':'.join(filter(bool, (self.namespace, self.url_name)))
 
     def __getitem__(self, index):
         return (self.func, self.args, self.kwargs)[index]
@@ -71,12 +68,16 @@ class ResolverMatch(object):
         return "ResolverMatch(func=%s, args=%s, kwargs=%s, url_name='%s', app_name='%s', namespace='%s')" % (
             self.func, self.args, self.kwargs, self.url_name, self.app_name, self.namespace)
 
+
 class Resolver404(Http404):
     pass
 
+
 class NoReverseMatch(Exception):
     pass
 
+
+@lru_cache.lru_cache(maxsize=None)
 def get_callable(lookup_view, can_fail=False):
     """
     Convert a string version of a function name to the callable object.
@@ -116,23 +117,24 @@ def get_callable(lookup_view, can_fail=False):
                         "Could not import %s. View does not exist in module %s." %
                         (lookup_view, mod_name))
     return lookup_view
-get_callable = memoize(get_callable, _callable_cache, 1)
 
+
+@lru_cache.lru_cache(maxsize=None)
 def get_resolver(urlconf):
     if urlconf is None:
         from django.conf import settings
         urlconf = settings.ROOT_URLCONF
     return RegexURLResolver(r'^/', urlconf)
-get_resolver = memoize(get_resolver, _resolver_cache, 1)
 
+
+@lru_cache.lru_cache(maxsize=None)
 def get_ns_resolver(ns_pattern, resolver):
     # Build a namespaced resolver for the given parent urlconf pattern.
     # This makes it possible to have captured parameters in the parent
     # urlconf pattern.
-    ns_resolver = RegexURLResolver(ns_pattern,
-                                          resolver.url_patterns)
+    ns_resolver = RegexURLResolver(ns_pattern, resolver.url_patterns)
     return RegexURLResolver(r'^/', [ns_resolver])
-get_ns_resolver = memoize(get_ns_resolver, _ns_resolver_cache, 2)
+
 
 def get_mod_func(callback):
     # Converts 'django.views.news.stories.story_detail' to
@@ -141,7 +143,8 @@ def get_mod_func(callback):
         dot = callback.rindex('.')
     except ValueError:
         return callback, ''
-    return callback[:dot], callback[dot+1:]
+    return callback[:dot], callback[dot + 1:]
+
 
 class LocaleRegexProvider(object):
     """
@@ -156,7 +159,6 @@ class LocaleRegexProvider(object):
         self._regex = regex
         self._regex_dict = {}
 
-
     @property
     def regex(self):
         """
@@ -229,6 +231,7 @@ class RegexURLPattern(LocaleRegexProvider):
         self._callback = get_callable(self._callback_str)
         return self._callback
 
+
 class RegexURLResolver(LocaleRegexProvider):
     def __init__(self, regex, urlconf_name, default_kwargs=None, app_name=None, namespace=None):
         LocaleRegexProvider.__init__(self, regex)
@@ -268,6 +271,9 @@ class RegexURLResolver(LocaleRegexProvider):
                 self._callback_strs.add(pattern._callback_str)
             elif hasattr(pattern, '_callback'):
                 callback = pattern._callback
+                if isinstance(callback, functools.partial):
+                    callback = callback.func
+
                 if not hasattr(callback, '__name__'):
                     lookup_str = callback.__module__ + "." + callback.__class__.__name__
                 else:
@@ -282,12 +288,10 @@ class RegexURLResolver(LocaleRegexProvider):
                     if pattern.app_name:
                         apps.setdefault(pattern.app_name, []).append(pattern.namespace)
                 else:
-                    parent = normalize(pattern.regex.pattern)
+                    parent_pat = pattern.regex.pattern
                     for name in pattern.reverse_dict:
                         for matches, pat, defaults in pattern.reverse_dict.getlist(name):
-                            new_matches = []
-                            for piece, p_args in parent:
-                                new_matches.extend([(piece + suffix, p_args + args) for (suffix, args) in matches])
+                            new_matches = normalize(parent_pat + pat)
                             lookups.appendlist(name, (new_matches, p_pattern + pat, dict(defaults, **pattern.default_kwargs)))
                     for namespace, (prefix, sub_pattern) in pattern.namespace_dict.items():
                         namespaces[namespace] = (p_pattern + prefix, sub_pattern)
@@ -326,6 +330,7 @@ class RegexURLResolver(LocaleRegexProvider):
         return self._app_dict[language_code]
 
     def resolve(self, path):
+        path = force_text(path)  # path may be a reverse_lazy object
         tried = []
         match = self.regex.search(path)
         if match:
@@ -336,7 +341,7 @@ class RegexURLResolver(LocaleRegexProvider):
                 except Resolver404 as e:
                     sub_tried = e.args[0].get('tried')
                     if sub_tried is not None:
-                        tried.extend([[pattern] + t for t in sub_tried])
+                        tried.extend([pattern] + t for t in sub_tried)
                     else:
                         tried.append([pattern])
                 else:
@@ -346,7 +351,7 @@ class RegexURLResolver(LocaleRegexProvider):
                         return ResolverMatch(sub_match.func, sub_match.args, sub_match_dict, sub_match.url_name, self.app_name or sub_match.app_name, [self.namespace] + sub_match.namespaces)
                     tried.append([pattern])
             raise Resolver404({'tried': tried, 'path': new_path})
-        raise Resolver404({'path' : path})
+        raise Resolver404({'path': path})
 
     @property
     def urlconf_module(self):
@@ -358,11 +363,17 @@ class RegexURLResolver(LocaleRegexProvider):
 
     @property
     def url_patterns(self):
+        # urlconf_module might be a valid set of patterns, so we default to it
         patterns = getattr(self.urlconf_module, "urlpatterns", self.urlconf_module)
         try:
             iter(patterns)
         except TypeError:
-            raise ImproperlyConfigured("The included urlconf %s doesn't have any patterns in it" % self.urlconf_name)
+            msg = (
+                "The included urlconf '{name}' does not appear to have any "
+                "patterns in it. If you see valid patterns in the file then "
+                "the issue is probably caused by a circular import."
+            )
+            raise ImproperlyConfigured(msg.format(name=self.urlconf_name))
         return patterns
 
     def _resolve_special(self, view_type):
@@ -431,7 +442,11 @@ class RegexURLResolver(LocaleRegexProvider):
                 candidate_pat = prefix_norm.replace('%', '%%') + result
                 if re.search('^%s%s' % (prefix_norm, pattern), candidate_pat % candidate_subs, re.UNICODE):
                     candidate_subs = dict((k, urlquote(v)) for (k, v) in candidate_subs.items())
-                    return candidate_pat % candidate_subs
+                    url = candidate_pat % candidate_subs
+                    # Don't allow construction of scheme relative urls.
+                    if url.startswith('//'):
+                        url = '/%%2F%s' % url[2:]
+                    return url
         # lookup_view can be URL label, or dotted path, or callable, Any of
         # these can be passed in at the top, but callables are not friendly in
         # error messages.
@@ -447,6 +462,7 @@ class RegexURLResolver(LocaleRegexProvider):
                 "arguments '%s' not found. %d pattern(s) tried: %s" %
                              (lookup_view_s, args, kwargs, len(patterns), patterns))
 
+
 class LocaleRegexURLResolver(RegexURLResolver):
     """
     A URL resolver that always matches the active language code as URL prefix.
@@ -466,11 +482,13 @@ class LocaleRegexURLResolver(RegexURLResolver):
             self._regex_dict[language_code] = regex_compiled
         return self._regex_dict[language_code]
 
+
 def resolve(path, urlconf=None):
     if urlconf is None:
         urlconf = get_urlconf()
     return get_resolver(urlconf).resolve(path)
 
+
 def reverse(viewname, urlconf=None, args=None, kwargs=None, prefix=None, current_app=None):
     if urlconf is None:
         urlconf = get_urlconf()
@@ -529,13 +547,12 @@ def reverse(viewname, urlconf=None, args=None, kwargs=None, prefix=None, current
 
 reverse_lazy = lazy(reverse, str)
 
+
 def clear_url_caches():
-    global _resolver_cache
-    global _ns_resolver_cache
-    global _callable_cache
-    _resolver_cache.clear()
-    _ns_resolver_cache.clear()
-    _callable_cache.clear()
+    get_callable.cache_clear()
+    get_resolver.cache_clear()
+    get_ns_resolver.cache_clear()
+
 
 def set_script_prefix(prefix):
     """
@@ -545,6 +562,7 @@ def set_script_prefix(prefix):
         prefix += '/'
     _prefixes.value = prefix
 
+
 def get_script_prefix():
     """
     Returns the currently active script prefix. Useful for client code that
@@ -553,6 +571,7 @@ def get_script_prefix():
     """
     return getattr(_prefixes, "value", '/')
 
+
 def clear_script_prefix():
     """
     Unsets the script prefix for the current thread.
@@ -562,6 +581,7 @@ def clear_script_prefix():
     except AttributeError:
         pass
 
+
 def set_urlconf(urlconf_name):
     """
     Sets the URLconf for the current thread (overriding the default one in
@@ -573,6 +593,7 @@ def set_urlconf(urlconf_name):
         if hasattr(_urlconfs, "value"):
             del _urlconfs.value
 
+
 def get_urlconf(default=None):
     """
     Returns the root URLconf to use for the current thread if it has been
@@ -580,6 +601,7 @@ def get_urlconf(default=None):
     """
     return getattr(_urlconfs, "value", default)
 
+
 def is_valid_path(path, urlconf=None):
     """
     Returns True if the given path resolves against the default URL resolver,
