@@ -6,7 +6,9 @@ import tempfile
 import os
 
 from django.contrib.auth.models import User
-from django.contrib.contenttypes import generic
+from django.contrib.contenttypes.fields import (
+    GenericForeignKey, GenericRelation
+)
 from django.contrib.contenttypes.models import ContentType
 from django.core.files.storage import FileSystemStorage
 from django.db import models
@@ -39,6 +41,11 @@ class Article(models.Model):
     model_year.admin_order_field = 'date'
     model_year.short_description = ''
 
+    def model_year_reversed(self):
+        return self.date.year
+    model_year_reversed.admin_order_field = '-date'
+    model_year_reversed.short_description = ''
+
 
 @python_2_unicode_compatible
 class Book(models.Model):
@@ -116,19 +123,23 @@ class ModelWithStringPrimaryKey(models.Model):
 class Color(models.Model):
     value = models.CharField(max_length=10)
     warm = models.BooleanField(default=False)
+
     def __str__(self):
         return self.value
 
+
 # we replicate Color to register with another ModelAdmin
 class Color2(Color):
     class Meta:
         proxy = True
 
+
 @python_2_unicode_compatible
 class Thing(models.Model):
     title = models.CharField(max_length=20)
     color = models.ForeignKey(Color, limit_choices_to={'warm': True})
     pub_date = models.DateField(blank=True, null=True)
+
     def __str__(self):
         return self.title
 
@@ -138,6 +149,7 @@ class Actor(models.Model):
     name = models.CharField(max_length=50)
     age = models.IntegerField()
     title = models.CharField(max_length=50, null=True, blank=True)
+
     def __str__(self):
         return self.name
 
@@ -166,13 +178,39 @@ class Sketch(models.Model):
         return self.title
 
 
+def today_callable_dict():
+    return {"last_action__gte": datetime.datetime.today()}
+
+
+def today_callable_q():
+    return models.Q(last_action__gte=datetime.datetime.today())
+
+
+@python_2_unicode_compatible
+class Character(models.Model):
+    username = models.CharField(max_length=100)
+    last_action = models.DateTimeField()
+
+    def __str__(self):
+        return self.username
+
+
+@python_2_unicode_compatible
+class StumpJoke(models.Model):
+    variation = models.CharField(max_length=100)
+    most_recently_fooled = models.ForeignKey(Character, limit_choices_to=today_callable_dict, related_name="+")
+    has_fooled_today = models.ManyToManyField(Character, limit_choices_to=today_callable_q, related_name="+")
+
+    def __str__(self):
+        return self.variation
+
+
 class Fabric(models.Model):
     NG_CHOICES = (
         ('Textured', (
-                ('x', 'Horizontal'),
-                ('y', 'Vertical'),
-            )
-        ),
+            ('x', 'Horizontal'),
+            ('y', 'Vertical'),
+        )),
         ('plain', 'Smooth'),
     )
     surface = models.CharField(max_length=20, choices=NG_CHOICES)
@@ -200,6 +238,7 @@ class Persona(models.Model):
     accounts which inherit from a common accounts class.
     """
     name = models.CharField(blank=False, max_length=80)
+
     def __str__(self):
         return self.name
 
@@ -253,7 +292,7 @@ class Podcast(Media):
     release_date = models.DateField()
 
     class Meta:
-        ordering = ('release_date',) # overridden in PodcastAdmin
+        ordering = ('release_date',)  # overridden in PodcastAdmin
 
 
 class Vodcast(Media):
@@ -390,8 +429,8 @@ class Post(models.Model):
     title = models.CharField(max_length=100, help_text="Some help text for the title (with unicode ŠĐĆŽćžšđ)")
     content = models.TextField(help_text="Some help text for the content (with unicode ŠĐĆŽćžšđ)")
     posted = models.DateField(
-            default=datetime.date.today,
-            help_text="Some help text for the date (with unicode ŠĐĆŽćžšđ)"
+        default=datetime.date.today,
+        help_text="Some help text for the date (with unicode ŠĐĆŽćžšđ)"
     )
     public = models.NullBooleanField()
 
@@ -399,6 +438,13 @@ class Post(models.Model):
         return "Very awesome."
 
 
+# Proxy model to test overridden fields attrs on Post model so as not to
+# interfere with other tests.
+class FieldOverridePost(Post):
+    class Meta:
+        proxy = True
+
+
 @python_2_unicode_compatible
 class Gadget(models.Model):
     name = models.CharField(max_length=100)
@@ -425,7 +471,7 @@ class FunkyTag(models.Model):
     name = models.CharField(max_length=25)
     content_type = models.ForeignKey(ContentType)
     object_id = models.PositiveIntegerField()
-    content_object = generic.GenericForeignKey('content_type', 'object_id')
+    content_object = GenericForeignKey('content_type', 'object_id')
 
     def __str__(self):
         return self.name
@@ -436,7 +482,7 @@ class Plot(models.Model):
     name = models.CharField(max_length=100)
     team_leader = models.ForeignKey(Villain, related_name='lead_plots')
     contact = models.ForeignKey(Villain, related_name='contact_plots')
-    tags = generic.GenericRelation(FunkyTag)
+    tags = GenericRelation(FunkyTag)
 
     def __str__(self):
         return self.name
@@ -610,25 +656,31 @@ class PrePopulatedPostLargeSlug(models.Model):
     published = models.BooleanField(default=False)
     slug = models.SlugField(max_length=1000)
 
+
 class AdminOrderedField(models.Model):
     order = models.IntegerField()
     stuff = models.CharField(max_length=200)
 
+
 class AdminOrderedModelMethod(models.Model):
     order = models.IntegerField()
     stuff = models.CharField(max_length=200)
+
     def some_order(self):
         return self.order
     some_order.admin_order_field = 'order'
 
+
 class AdminOrderedAdminMethod(models.Model):
     order = models.IntegerField()
     stuff = models.CharField(max_length=200)
 
+
 class AdminOrderedCallable(models.Model):
     order = models.IntegerField()
     stuff = models.CharField(max_length=200)
 
+
 @python_2_unicode_compatible
 class Report(models.Model):
     title = models.CharField(max_length=100)
@@ -644,8 +696,9 @@ class MainPrepopulated(models.Model):
         max_length=20,
         choices=(('option one', 'Option One'),
                  ('option two', 'Option Two')))
-    slug1 = models.SlugField()
-    slug2 = models.SlugField()
+    slug1 = models.SlugField(blank=True)
+    slug2 = models.SlugField(blank=True)
+
 
 class RelatedPrepopulated(models.Model):
     parent = models.ForeignKey(MainPrepopulated)
@@ -667,6 +720,7 @@ class UnorderedObject(models.Model):
     name = models.CharField(max_length=255)
     bool = models.BooleanField(default=True)
 
+
 class UndeletableObject(models.Model):
     """
     Model whose show_delete in admin change_view has been disabled
@@ -674,20 +728,102 @@ class UndeletableObject(models.Model):
     """
     name = models.CharField(max_length=255)
 
+
+class UnchangeableObject(models.Model):
+    """
+    Model whose change_view is disabled in admin
+    Refs #20640.
+    """
+
+
 class UserMessenger(models.Model):
     """
     Dummy class for testing message_user functions on ModelAdmin
     """
 
+
 class Simple(models.Model):
     """
     Simple model with nothing on it for use in testing
     """
 
+
 class Choice(models.Model):
     choice = models.IntegerField(blank=True, null=True,
         choices=((1, 'Yes'), (0, 'No'), (None, 'No opinion')))
 
+
+class ParentWithDependentChildren(models.Model):
+    """
+    Issue #20522
+    Model where the validation of child foreign-key relationships depends
+    on validation of the parent
+    """
+    some_required_info = models.PositiveIntegerField()
+    family_name = models.CharField(max_length=255, blank=False)
+
+
+class DependentChild(models.Model):
+    """
+    Issue #20522
+    Model that depends on validation of the parent class for one of its
+    fields to validate during clean
+    """
+    parent = models.ForeignKey(ParentWithDependentChildren)
+    family_name = models.CharField(max_length=255)
+
+
+class _Manager(models.Manager):
+    def get_queryset(self):
+        return super(_Manager, self).get_queryset().filter(pk__gt=1)
+
+
+class FilteredManager(models.Model):
+    def __str__(self):
+        return "PK=%d" % self.pk
+
+    pk_gt_1 = _Manager()
+    objects = models.Manager()
+
+
+class EmptyModelVisible(models.Model):
+    """ See ticket #11277. """
+
+
+class EmptyModelHidden(models.Model):
+    """ See ticket #11277. """
+
+
+class EmptyModelMixin(models.Model):
+    """ See ticket #11277. """
+
+
+class State(models.Model):
+    name = models.CharField(max_length=100)
+
+
+class City(models.Model):
+    state = models.ForeignKey(State)
+    name = models.CharField(max_length=100)
+
+    def get_absolute_url(self):
+        return '/dummy/%s/' % self.pk
+
+
+class Restaurant(models.Model):
+    city = models.ForeignKey(City)
+    name = models.CharField(max_length=100)
+
+    def get_absolute_url(self):
+        return '/dummy/%s/' % self.pk
+
+
+class Worker(models.Model):
+    work_at = models.ForeignKey(Restaurant)
+    name = models.CharField(max_length=50)
+    surname = models.CharField(max_length=50)
+
+
 # Models for #23329
 class ReferencedByParent(models.Model):
     pass
@@ -703,4 +839,3 @@ class ChildOfReferer(ParentWithFK):
 
 class M2MReference(models.Model):
     ref = models.ManyToManyField('self')
-
